/*
 *    LS App - LoadSensing Application - https://github.com/SergiP/LS-App
 *    
 *    Copyright (C) 2011-2012
 *    Authors:
 *    	Sergio Gonz�lez D�ez        [sergio.gd@gmail.com]
 *    	Sergio Postigo Collado      [spostigoc@gmail.com]
 *    
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *    
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *    
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package com.lsn.LoadSensing;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.lsn.LoadSensing.actionbar.ActionBarActivity;
import com.lsn.LoadSensing.element.LSNetwork;
import com.lsn.LoadSensing.func.LSFunctions;
import com.lsn.LoadSensing.ui.CustomToast;
import com.readystatesoftware.mapviewballoons.R;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;

public class LSHomeActivity extends ActionBarActivity {

	ArrayList<LSNetwork> m_networks = null;

	private String typeMaps = null;
	public static String idSession;
	private Intent i = null;

	private JSONObject jsonData = null;
	private JSONArray jArray = null;

	@Override
	public void onCreate(Bundle savedInstanceState) {

		super.onCreate(savedInstanceState);
		setContentView(R.layout.dshb_home);

		// Change home icon (<Icon)
		getActionBarHelper().changeIconHome();

		// Retrieve intent information
		Bundle bundle = getIntent().getExtras();

		if (bundle != null) {
			idSession = bundle.getString("SESSION");
			CustomToast.showCustomToast(
					this,
					this.getString(R.string.msg_Welcome) + " "
							+ bundle.getString("USER") + " " + idSession,
					CustomToast.IMG_CORRECT, CustomToast.LENGTH_LONG);
		}

		// Set click Listener on dashboard buttons
		DashboardClickListener dbClickListener = new DashboardClickListener();
		findViewById(R.id.dsh_btn_netList).setOnClickListener(dbClickListener);
		findViewById(R.id.dsh_btn_netMaps).setOnClickListener(dbClickListener);
		findViewById(R.id.dsh_btn_QRCode).setOnClickListener(dbClickListener);
		findViewById(R.id.dsh_btn_Faves).setOnClickListener(dbClickListener);
		findViewById(R.id.dsh_btn_AR).setOnClickListener(dbClickListener);
		findViewById(R.id.dsh_btn_netCloser)
				.setOnClickListener(dbClickListener);
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {

		MenuInflater menuInflater = getMenuInflater();
		menuInflater.inflate(R.menu.ab_item_ov_info_help_config, menu);

		getActionBarHelper().optionsMenuInfo(menu);
		getActionBarHelper().optionsMenuConfig(menu);
		getActionBarHelper().optionsMenuHelp(menu);

		return super.onCreateOptionsMenu(menu);
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {

		switch (item.getItemId()) {

		case android.R.id.home:
			showLogOutDialog();
			break;

		case R.id.menu_info:
			i = new Intent(LSHomeActivity.this, LSInfoActivity.class);
			break;

		case R.id.menu_config:
			i = new Intent(LSHomeActivity.this, LSConfigActivity.class);
			break;

		case R.id.menu_help:
			i = new Intent(LSHomeActivity.this, LSHelpActivity.class);
			break;
		}

		if (i != null) {
			startActivity(i);
		}

		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onResume() {

		super.onResume();

		SharedPreferences settings = PreferenceManager
				.getDefaultSharedPreferences(this);
		typeMaps = settings.getString("maps", "google");
	}

	private class DashboardClickListener implements OnClickListener {

		public void onClick(View v) {

			switch (v.getId()) {
			case R.id.dsh_btn_netList:
				i = new Intent(LSHomeActivity.this, LSNetListActivity.class);
				break;

			case R.id.dsh_btn_netMaps:
				ProgressDialog progressDialog = new ProgressDialog(
						LSHomeActivity.this);
				progressDialog.setTitle(getString(R.string.msg_PleaseWait));
				progressDialog
						.setMessage(getString(R.string.msg_retrievNetworks));
				progressDialog.setCancelable(true);

				NetListTask netListTask = new NetListTask(LSHomeActivity.this,
						progressDialog);
				netListTask.execute();
				break;

			case R.id.dsh_btn_QRCode:
				i = new Intent(LSHomeActivity.this, LSQRCodeActivity.class);
				break;

			case R.id.dsh_btn_Faves:
				Bundle bundle = new Bundle();
				bundle.putInt("par", 0);

				i = new Intent(LSHomeActivity.this, LSFavesActivity.class);
				i.putExtras(bundle);
				break;

			case R.id.dsh_btn_AR:
				ProgressDialog progressDialog2 = new ProgressDialog(
						LSHomeActivity.this);
				progressDialog2.setTitle(getString(R.string.msg_PleaseWait));
				progressDialog2
						.setMessage(getString(R.string.msg_retrievNetworks));
				progressDialog2.setCancelable(false);

				AugRealTask augReal = new AugRealTask(LSHomeActivity.this,
						progressDialog2);
				augReal.execute();
				break;

			case R.id.dsh_btn_netCloser:
				i = new Intent(LSHomeActivity.this, LSNetCloserActivity.class);
				break;

			default:
				break;

			}

			if (i != null) {
				startActivity(i);
			}
		}
	}

	@Override
	public void onBackPressed() {

		showLogOutDialog();
	}

	private void showLogOutDialog() {

		AlertDialog.Builder dialogLogOut = new AlertDialog.Builder(this);
		dialogLogOut
				.setMessage(R.string.dialogLogOut)
				.setPositiveButton(R.string.txtYes,
						new DialogInterface.OnClickListener() {

							public void onClick(DialogInterface dialog, int id) {

								// Go to Login Activity erasing all other
								// activities with FLAG_ACTIVITY_CLEAR_TOP
								Intent i = new Intent(LSHomeActivity.this,
										LSLoginActivity.class)
										.setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP);
								startActivity(i);
							}
						})
				.setNegativeButton(R.string.txtNo,
						new DialogInterface.OnClickListener() {

							public void onClick(DialogInterface dialog, int id) {

								dialog.cancel();
							}
						});
		AlertDialog alertLogOut = dialogLogOut.create();
		alertLogOut.setTitle(R.string.txtExit);
		alertLogOut.show();
	}

	public class NetListTask extends AsyncTask<String, Void, String> {

		@SuppressWarnings("unused")
		private Activity activity;
		private ProgressDialog progressDialog;
		private String messageReturn = null;

		public NetListTask(Activity activity, ProgressDialog progressDialog) {

			this.progressDialog = progressDialog;
			this.activity = activity;
		}

		@Override
		protected void onPreExecute() {

			progressDialog.show();
		}

		@Override
		protected String doInBackground(String... arg0) {

			m_networks = new ArrayList<LSNetwork>();
			// Server Request Ini
			Map<String, String> params = new HashMap<String, String>();
			params.put("session", LSHomeActivity.idSession);
			jArray = LSFunctions.urlRequestJSONArray(
					"http://viuterrassa.com/Android/getLlistatXarxes.php",
					params);

			return messageReturn;
		}

		@Override
		protected void onPostExecute(String pMessageReturn) {

			progressDialog.dismiss();

			goListMaps();
		}
	}

	public void goListMaps() {

		if (jArray != null) {
			try {
				for (int i = 0; i < jArray.length(); i++) {
					jsonData = jArray.getJSONObject(i);
					LSNetwork network = new LSNetwork();
					network.setNetworkName(jsonData.getString("Nom"));
					network.setNetworkPosition(jsonData.getString("Lat"),
							jsonData.getString("Lon"));
					network.setNetworkNumSensors(jsonData.getString("Sensors"));
					network.setNetworkId(jsonData.getString("IdXarxa"));
					network.setNetworkSituation(jsonData.getString("Poblacio"));
					m_networks.add(network);
				}
			} catch (JSONException e) {
				CustomToast.showCustomToast(LSHomeActivity.this,
						R.string.msg_ProcessError, CustomToast.IMG_AWARE,
						CustomToast.LENGTH_SHORT);
			}
		} else {
			CustomToast.showCustomToast(LSHomeActivity.this,
					R.string.msg_CommError, CustomToast.IMG_AWARE,
					CustomToast.LENGTH_SHORT);
		}

		if (typeMaps.equals("google")) {
			i = new Intent(LSHomeActivity.this, LSNetMapsActivity.class);
		} else {
			i = new Intent(LSHomeActivity.this, LSNetMapsForgeActivity.class);
		}

		Bundle bundle = new Bundle();
		bundle.putParcelableArrayList("NETWORKS", m_networks);
		i.putExtras(bundle);
		startActivity(i);
	}

	public class AugRealTask extends AsyncTask<String, Void, String> {

		@SuppressWarnings("unused")
		private Activity activity;
		private ProgressDialog progressDialog;
		private String messageReturn = null;

		public AugRealTask(Activity activity, ProgressDialog progressDialog) {

			this.progressDialog = progressDialog;
			this.activity = activity;
		}

		@Override
		protected void onPreExecute() {

			progressDialog.show();
		}

		@Override
		protected String doInBackground(String... arg0) {

			m_networks = new ArrayList<LSNetwork>();
			// Server Request Ini
			Map<String, String> params = new HashMap<String, String>();
			params.put("session", LSHomeActivity.idSession);
			jArray = LSFunctions.urlRequestJSONArray(
					"http://viuterrassa.com/Android/getLlistatXarxes.php",
					params);

			return messageReturn;
		}

		@Override
		protected void onPostExecute(String pMessageReturn) {

			progressDialog.dismiss();

			goAugReal();
		}
	}

	public void goAugReal() {

		if (jArray != null) {
			i = new Intent(LSHomeActivity.this, LSAugRealActivity.class);
			i.putExtra("ARRAY", jArray.toString());

			startActivity(i);
		}
	}
}